import logging
import typing

from ..schema import Scan, SchedulingOptions
from ..typehints import (
    CallbackFunction,
    InputReportTemplate,
    InputScan,
    InputScanProfile,
    InputTargetType,
)
from ..utils import (
    get_input_report_template_id,
    get_input_scan_id,
    get_input_scan_profile_id,
    get_input_target_id,
)


class Scans:
    async def get_scans(self) -> typing.List[Scan]:
        """
        Get all scans
        :return: List of `Scan` objects
        :example:
        ```python
            >>> scans = await api.scans.get_scans()
            >>> for scan in scans:
            >>>     print(scan)
        ```
        """
        return [
            Scan.from_dict(scan)
            for scan in (await self._request("GET", "scans"))["scans"]
        ]

    async def get_scan(self, scan: InputScan) -> Scan:
        """
        Get scan by ID
        :param scan_id: Scan ID
        :return: `Scan` object
        :example:
        ```python
            >>> scan = await api.scans.get_scan(
                    "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                )
            >>> print(scan)
            >>> scan = await api.scans.get_scan(
                    Scan(...),
                )
            >>> print(scan)
        ```
        """
        return Scan.from_dict(
            await self._request("GET", f"scans/{get_input_scan_id(scan)}")
        )

    async def create_scan(
        self,
        target: InputTargetType,
        profile: InputScanProfile,
        report_template: typing.Optional[InputReportTemplate] = None,
        schedule: typing.Optional[SchedulingOptions] = None,
        done_callback: typing.Optional[CallbackFunction] = None,
    ) -> Scan:
        """
        Create new scan
        :param target: Target ID or `Target` object
        :param profile: Profile ID or `ScanProfile` object
        :param report_template: Report template ID or `ReportTemplate` object
        :param schedule: Scheduling options
        :param done_callback: Callback function that will be called
                              when scan is finished (not report ready!)
        :return: `Scan` object
        :example:
        ```python
            >>> scan = await api.scans.create_scan(
                    "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                    "11111111-1111-1111-1111-111111111111",
                )
            >>> print(scan)
        ```
        """

        if not schedule:
            schedule = SchedulingOptions(
                disable=False,
                history_limit=0,
                recurrence=0,
                start_date=None,
                time_sensitive=False,
                triggerable=False,
            )

        response: dict = await self._request(
            "POST",
            "scans",
            {
                "profile_id": get_input_scan_profile_id(profile),
                "target_id": get_input_target_id(target),
                "schedule": schedule,
                **(
                    {
                        "report_template_id": get_input_report_template_id(
                            report_template
                        )
                    }
                    if report_template
                    else {}
                ),
            },
        )

        if "scan_id" not in response:
            raise ValueError("Invalid response")

        if done_callback:
            self._callbacks[response["scan_id"]] = done_callback

        return await self.get_scan(response["scan_id"])

    async def delete_scan(self, scan: InputScan) -> None:
        """
        Delete scan by ID
        :param scan_id: Scan ID
        :example:
        ```python
            >>> await api.scans.delete_scan(
                    "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                )
            >>> await api.scans.delete_scan(Scan(...))
        ```
        """
        await self._request("DELETE", f"scans/{get_input_scan_id(scan)}")

    async def pause_scan(self, scan: InputScan) -> None:
        """
        Pause scan by ID
        :param scan_id: Scan ID
        :example:
        ```python
            >>> await api.scans.pause_scan(
                    "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                )
            >>> await api.scans.pause_scan(Scan(...))
        ```
        """
        await self._request("POST", f"scans/{get_input_scan_id(scan)}/pause", {})

    async def resume_scan(self, scan: InputScan) -> None:
        """
        Resume scan by ID
        :param scan_id: Scan ID
        :example:
        ```python
            >>> await api.scans.resume_scan(
                    "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                )
            >>> await api.scans.resume_scan(Scan(...))
        ```
        """
        await self._request("POST", f"scans/{get_input_scan_id(scan)}/resume", {})

    async def stop_scan(self, scan: InputScan) -> None:
        """
        Stop scan by ID
        :param scan_id: Scan ID
        :example:
        ```python
            >>> await api.scans.stop_scan(
                    "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                )
            >>> await api.scans.stop_scan(Scan(...))
        ```
        """
        await self._request("POST", f"scans/{get_input_scan_id(scan)}/abort", {})



from . import (
    access_map,
    current_session,
    file_attachment,
    input_target,
    notification,
    notification_data,
    report,
    report_template,
    scan,
    scan_profile,
    scheduling_options,
    target,
    typed_list,
    user,
)
from .access_map import AccessMap
from .current_session import CurrentSession
from .file_attachment import FileAttachment
from .input_target import InputTarget
from .notification import Notification
from .notification_data import NotificationData
from .report import Report
from .report_template import (
    AFFECTED_ITEMS,
    CWE_SANS_TOP_25,
    DEVELOPER,
    EXECUTIVE_SUMMARY,
    HIPAA,
    ISO_27001,
    NIST_SP800_53,
    OWASP_TOP_10_2013,
    OWASP_TOP_10_2017,
    OWASP_TOP_10_2021,
    PCI_DSS_3_2,
    QUICK,
    SARBANES_OXLEY,
    SCAN_COMPARISON,
    STIG_DISA,
    TEMPLATE_MAP,
    WASC_THREAT_CLASSIFICATION,
    ReportTemplate,
)
from .scan import Scan
from .scan_profile import (
    CRAWL_ONLY,
    CROSS_SITE_SCRIPTING_VULNERABILITIES,
    FULL_SCAN,
    HIGH_OR_MEDIUM_RISK_VULNERABILITIES,
    HIGH_RISK_VULNERABILITIES,
    MALWARE_SCAN,
    SQL_INJECTION_VULNERABILITIES,
    WEAK_PASSWORDS,
    ScanProfile,
)
from .scheduling_options import SchedulingOptions
from .target import Target
from .typed_list import TypedList
from .user import User

__all__ = [
    "CurrentSession",
    "FileAttachment",
    "InputTarget",
    "Notification",
    "NotificationData",
    "Report",
    "Scan",
    "SchedulingOptions",
    "Target",
    "TypedList",
    "User",
    "AccessMap",
    "ReportTemplate",
    "DEVELOPER",
    "QUICK",
    "EXECUTIVE_SUMMARY",
    "HIPAA",
    "AFFECTED_ITEMS",
    "SCAN_COMPARISON",
    "CWE_SANS_TOP_25",
    "ISO_27001",
    "NIST_SP800_53",
    "OWASP_TOP_10_2013",
    "OWASP_TOP_10_2017",
    "OWASP_TOP_10_2021",
    "PCI_DSS_3_2",
    "SARBANES_OXLEY",
    "STIG_DISA",
    "WASC_THREAT_CLASSIFICATION",
    "TEMPLATE_MAP",
    "ScanProfile",
    "FULL_SCAN",
    "HIGH_RISK_VULNERABILITIES",
    "HIGH_OR_MEDIUM_RISK_VULNERABILITIES",
    "CROSS_SITE_SCRIPTING_VULNERABILITIES",
    "SQL_INJECTION_VULNERABILITIES",
    "WEAK_PASSWORDS",
    "CRAWL_ONLY",
    "MALWARE_SCAN",
    "access_map",
    "current_session",
    "file_attachment",
    "input_target",
    "notification",
    "notification_data",
    "report",
    "report_template",
    "scan",
    "scan_profile",
    "scheduling_options",
    "target",
    "typed_list",
    "user",
]



import typing

from .schema import Report, ReportTemplate, Scan, ScanProfile, Target, User

InputReport = typing.Union[Report, str]
InputScan = typing.Union[Scan, str]
InputTargetType = typing.Union[Target, str]
InputReportTemplate = typing.Union[ReportTemplate, str]
InputUser = typing.Union[User, str]
InputScanProfile = typing.Union[ScanProfile, str]
CallbackFunction = typing.Union[typing.Callable, typing.Callable[..., typing.Awaitable]]



from . import reports, scans, targets, users
from .reports import Reports
from .scans import Scans
from .targets import Targets
from .users import Users

__all__ = [
    "Reports",
    "Scans",
    "Targets",
    "Users",
    "reports",
    "scans",
    "targets",
    "users",
]



SCAN_TYPE_MAP = {
    205: "Target not Responsive",
    208: "Target Verification Failed",
    209: "Target Continuous Mode Disabled",
    601: "Report Created",
    604: "Report Failed",
    605: "Export Failed",
    400: "Scan Scheduled",
    401: "Scan Modified",
    402: "Scan Deleted",
    403: "Scan Started",
    404: "Scan Done",
    405: "Scan Failed",
    406: "Scan User Aborted",
    407: "Scan Recurrence Suspended",
    410: "Scan Job Starting",
    411: "Scan Job Completed",
    412: "Scan Job Failed",
    413: "Scan Job Aborted",
    415: "Scan Paused",
    420: "Scan Scanner Event",
    430: "Scan Crawler Memory Limited Reached",
    431: "Scan Aborted By Scanner",
    432: "Scan AcuSensor Found",
    433: "Scan AcuSensor Not Found",
    434: "Scan Automatic Login Failed",
    435: "Scan Invalid HTTP LoginDto",
    436: "Scan Login Sequence Required",
    437: "Scan Crawling",
    438: "Scan Deepscan",
    439: "Scan Started",
    440: "Scan Finished",
    441: "Scan Manual Intervention",
    442: "Scan Resumed",
    443: "Initial Request",
    444: "Importer Failed",
    445: "Target not responsive",
    446: "LSR/Autologin not configured",
    502: "Vulnerability discovered by AcuMonitor",
    801: "Engine is not responding",
    802: "Engine is offline",
    803: "Engine is back online",
}

SCAN_DONE_STATUS = [404, 405, 406, 407, 412, 413, 431, 440, 445]
REPORT_DONE_STATUS = [601, 604]
EXPORT_DONE_STATUS = [605]



import typing

from ..schema import InputTarget, Target
from ..utils import get_input_target_id


class Targets:
    async def get_targets(self) -> typing.List[Target]:
        """
        Get all targets
        :return: List of `Target` objects
        :example:
        ```python
            >>> targets = await api.targets.get_targets()
            >>> for target in targets:
            >>>     print(target)
        ```
        """
        return [
            Target.from_dict(target)
            for target in (await self._request("GET", "targets"))["targets"]
        ]

    async def get_target(self, target: typing.Union[Target, str]) -> Target:
        """
        Get target by ID
        :param target: Target ID
        :return: `Target` object
        :example:
        ```python
            >>> target = await api.targets.get_target(
                    "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                )
            >>> print(target)
            >>> target = await api.targets.get_target(
                    Target(...),
                )
            >>> print(target)
        ```
        """
        return Target.from_dict(
            await self._request("GET", f"targets/{get_input_target_id(target)}")
        )

    async def create_targets(
        self,
        targets: typing.List[InputTarget],
        groups: typing.Optional[typing.List[str]] = None,
    ) -> Target:
        """
        Create new targets
        :param targets: List of targets
        :param groups: List of groups
        :return: List of `Target` objects
        :example:
        ```python
            >>> targets = await api.targets.create_targets(
                    [
                        InputTarget(
                            address="https://example.com",
                            description="Example",
                        ),
                        InputTarget(
                            address="https://example.org",
                            description="Example",
                        ),
                    ],
                )
            >>> for target in targets:
            >>>     print(target)
        ```
        """
        response: dict = await self._request(
            "POST", "targets/add", {"targets": targets, "groups": groups or []}
        )

        return [
            await self.get_target(target["target_id"]) for target in response["targets"]
        ]

    async def delete_targets(self, targets: typing.List[typing.Union[Target, str]]):
        """
        Delete targets
        :param targets: List of targets
        :return: None
        :example:
        ```python
            >>> await api.targets.delete_targets(
                    [
                        "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                        Target(...),
                    ],
                )
        ```
        """
        await self._request(
            "POST",
            "targets/delete",
            {"targets": [get_input_target_id(target) for target in targets]},
        )

    async def delete_target(self, target: typing.Union[Target, str]):
        """
        Delete target
        :param target: Target
        :return: None
        :example:
        ```python
            >>> await api.targets.delete_target(
                    "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                )
            >>> await api.targets.delete_target(
                    Target(...),
                )
        ```
        """
        await self.delete_targets([target])

    async def create_target(
        self,
        target: InputTarget,
        groups: typing.Optional[typing.List[str]] = None,
    ) -> Target:
        """
        Create new target
        :param target: Target
        :param groups: List of groups
        :return: `Target` object
        :example:
        ```python
            >>> target = await api.targets.create_target(
                    InputTarget(
                        address="https://example.com",
                        description="Example",
                    ),
                )
            >>> print(target)
        ```
        """
        return (await self.create_targets([target], groups))[0]



import asyncio
import inspect
import io
import json
import logging
import string
import typing

import aiohttp

from . import errors, methods, schema, status_codes
from .errors import (
    ERROR_MAP,
    Acunetix400Error,
    Acunetix401Error,
    Acunetix403Error,
    Acunetix404Error,
    Acunetix409Error,
    Acunetix422Error,
    Acunetix429Error,
    Acunetix500Error,
    Acunetix502Error,
    Acunetix503Error,
    Acunetix504Error,
    AcunetixAPIError,
)
from .methods import Reports, Scans, Targets
from .schema import (
    AFFECTED_ITEMS,
    CRAWL_ONLY,
    CROSS_SITE_SCRIPTING_VULNERABILITIES,
    CWE_SANS_TOP_25,
    DEVELOPER,
    EXECUTIVE_SUMMARY,
    FULL_SCAN,
    HIGH_OR_MEDIUM_RISK_VULNERABILITIES,
    HIGH_RISK_VULNERABILITIES,
    HIPAA,
    ISO_27001,
    MALWARE_SCAN,
    NIST_SP800_53,
    OWASP_TOP_10_2013,
    OWASP_TOP_10_2017,
    OWASP_TOP_10_2021,
    PCI_DSS_3_2,
    QUICK,
    SARBANES_OXLEY,
    SCAN_COMPARISON,
    SQL_INJECTION_VULNERABILITIES,
    STIG_DISA,
    TEMPLATE_MAP,
    WASC_THREAT_CLASSIFICATION,
    WEAK_PASSWORDS,
    AccessMap,
    FileAttachment,
    InputTarget,
    Notification,
    NotificationData,
    Report,
    ReportTemplate,
    Scan,
    Target,
    TypedList,
    User,
)
from .status_codes import EXPORT_DONE_STATUS, REPORT_DONE_STATUS, SCAN_DONE_STATUS
from .typehints import (
    CallbackFunction,
    InputReport,
    InputReportTemplate,
    InputScan,
    InputScanProfile,
    InputTargetType,
    InputUser,
)

__all__ = [
    "AcunetixAPI",
    "AcunetixAPIError",
    "Acunetix400Error",
    "Acunetix401Error",
    "Acunetix403Error",
    "Acunetix404Error",
    "Acunetix409Error",
    "Acunetix422Error",
    "Acunetix429Error",
    "Acunetix500Error",
    "Acunetix502Error",
    "Acunetix503Error",
    "Acunetix504Error",
    "ERROR_MAP",
    "EXPORT_DONE_STATUS",
    "FileAttachment",
    "InputTarget",
    "Notification",
    "NotificationData",
    "REPORT_DONE_STATUS",
    "Report",
    "SCAN_DONE_STATUS",
    "Scan",
    "Target",
    "status_codes",
    "methods",
    "schema",
    "errors",
    "TypedList",
    "User",
    "AccessMap",
    "ReportTemplate",
    "DEVELOPER",
    "QUICK",
    "EXECUTIVE_SUMMARY",
    "HIPAA",
    "AFFECTED_ITEMS",
    "SCAN_COMPARISON",
    "CWE_SANS_TOP_25",
    "ISO_27001",
    "NIST_SP800_53",
    "OWASP_TOP_10_2013",
    "OWASP_TOP_10_2017",
    "PCI_DSS_3_2",
    "SARBANES_OXLEY",
    "STIG_DISA",
    "WASC_THREAT_CLASSIFICATION",
    "TEMPLATE_MAP",
    "InputReport",
    "InputScan",
    "InputTargetType",
    "InputReportTemplate",
    "InputUser",
    "CallbackFunction",
    "FULL_SCAN",
    "HIGH_RISK_VULNERABILITIES",
    "HIGH_OR_MEDIUM_RISK_VULNERABILITIES",
    "CROSS_SITE_SCRIPTING_VULNERABILITIES",
    "SQL_INJECTION_VULNERABILITIES",
    "WEAK_PASSWORDS",
    "CRAWL_ONLY",
    "MALWARE_SCAN",
    "InputScanProfile",
]

logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)


# pylint: disable=unused-argument
def _default(self, obj):  # skipcq: PYL-W0613
    """
    Patch default JSON serializer to support custom classes
    It will call `to_json` method on the object if it exists
    """
    return getattr(obj.__class__, "to_json", _default.default)(obj)


_default.default = json.JSONEncoder().default
json.JSONEncoder.default = _default


class AcunetixAPI(Scans, Targets, Reports):
    """
    Acunetix API client.
    You must deploy your own Acunetix instance to use this client.
    """

    def __init__(self, api_key: str, endpoint: str = "localhost:3443"):
        """
        :param api_key: API key from Acunetix
        :param endpoint: Acunetix API endpoint. Default is localhost:3443
        :example:
        ```python
            >>> api = AcunetixAPI(
                    "1986ad8c0a5b3df4d7028d5f3c06e946cdcc13b0d336841bc93daf7e58fb8a63a",
                )
        ```
        """
        if not api_key or any(char not in string.hexdigits for char in api_key):
            raise ValueError("Invalid API key")

        if not endpoint:
            raise ValueError("Invalid endpoint")

        self._api_key: str = api_key
        self._endpoint: str = endpoint
        self._callbacks: dict = {}
        self._connected: bool = False

    async def connect(self, no_credentials_check: bool = False):
        """
        Connect to Acunetix API. Required before using any other methods.
        Without calling it, the credentials will be checked only when the
        first request is made. Also, callbacks will not be executed.
        :example:
        ```python
            >>> await api.connect()
            ```
        """
        if self._connected:
            return

        self._connected = True

        logger.debug("Connecting...")
        if not no_credentials_check:
            await self._check_credentials()

        logger.debug("Connected")

        asyncio.create_task(self._poll_notifications())

    async def _execute_callback(self, callback_id: str, notification: Notification):
        """
        Executes a callback if it exists
        :param callback_id: Callback ID
        :param notification: Notification
        """
        if callback_id not in self._callbacks:
            return

        logger.debug("Executing callback for %s", callback_id)

        callback: typing.Callable = self._callbacks.pop(callback_id)
        callback: typing.Union[typing.Awaitable, typing.Any] = (
            callback(notification)
            if inspect.signature(callback).parameters
            else callback()
        )

        if asyncio.iscoroutinefunction(callback):
            await callback

    async def _poll_notifications(self):
        """Polls the Acunetix API for notifications"""
        while True:
            try:
                notifications: dict = await self._request("GET", "notifications")
                for notification in notifications["notifications"]:
                    logger.debug(
                        "Processing notification %s",
                        notification["notification_id"],
                    )
                    if notification["type_id"] in SCAN_DONE_STATUS + REPORT_DONE_STATUS:
                        logger.debug(
                            "Executing callback for %s",
                            notification["resource_id"],
                        )
                        await self._execute_callback(
                            notification["resource_id"],
                            Notification.from_dict(notification),
                        )

                    await self._request(
                        "POST",
                        f"notifications/{notification['notification_id']}/consume",
                    )

            except AcunetixAPIError as e:
                logger.exception(e)

            await asyncio.sleep(5)

    async def _request(
        self,
        method: str,
        path: str,
        data: dict = None,
        full_response: bool = False,
    ) -> typing.Union[dict, aiohttp.ClientResponse]:
        """
        Makes a request to the Acunetix API
        :param method: HTTP method
        :param path: API path
        :param data: JSON data
        :param full_response: If True, returns the full response object
        :return: JSON response or full response object
        """
        logger.debug("Requesting %s %s", method, path)
        async with aiohttp.ClientSession() as session:
            async with session.request(
                method,
                f"https://{self._endpoint}/api/v1/{path}",
                json=data,
                headers={"X-Auth": self._api_key},
            ) as response:
                if response.status == 204:
                    return {}

                if response.status not in range(200, 300):
                    raise ERROR_MAP.get(response.status, AcunetixAPIError)(
                        response.status,
                        await response.text(),
                    )

                return response if full_response else await response.json()

    async def _check_credentials(self):
        """Checks if the credentials are valid"""
        await self._request("GET", "me")

    async def default_scan(
        self,
        target: InputTarget,
        scan_profile: InputScanProfile = FULL_SCAN,
        report_template: InputReportTemplate = DEVELOPER,
    ) -> typing.List[io.BytesIO]:
        """
        Performs a default scan on a target
        :param target: Target
        :param scan_profile: Scan profile
        :param report_template: Report template
        :return: List of reports as PDF files objects
        :example:
        ```python
            >>> import os
            >>> target = InputTarget(
                    target="http://example.com",
                    description="Example target",
                )
            >>> files = await api.default_scan(target)
            >>> for file in files:
            >>>     with open(os.path.join("reports", file.name), "wb") as f:
            >>>         f.write(file.read())
            >>> print(os.listdir("reports"))
        ```
        """
        logger.debug("Creating target %s", target)
        target: Target = await self.create_target(target)
        done_event: asyncio.Event = asyncio.Event()

        logger.debug("Performing scan on %s", target.target_id)
        scan: Scan = await self.create_scan(
            target,
            scan_profile,
            # If we generate report here, we won't be able to attach callback
            # to it. This is how Acunetix works. That's why we omit
            # default report creation here and create it manually later.
            None,
            None,
            done_event.set,
        )
        await done_event.wait()

        logger.debug("Retrieving scan results %s", scan.scan_id)
        scan = await self.get_scan(scan)
        done_event.clear()

        logger.debug("Creating report for %s", scan.scan_id)
        report: Report = await self.create_report(
            report_template,
            TypedList("scans", [scan.scan_id]),
            done_event.set,
        )
        await done_event.wait()

        logger.debug("Retrieving report %s", report.report_id)
        files: typing.List[io.BytesIO] = await self.download_report(report)

        logger.debug("Report ready %s", report.report_id)
        return files



class AcunetixAPIError(Exception):
    def __init__(self, status: int, message: str):
        self.status = status
        self.message = message

    def __str__(self):
        return (
            f"Got response with status code {self.status} from Acunetix: {self.message}"
        )


class Acunetix404Error(AcunetixAPIError):
    pass


class Acunetix401Error(AcunetixAPIError):
    pass


class Acunetix400Error(AcunetixAPIError):
    pass


class Acunetix500Error(AcunetixAPIError):
    pass


class Acunetix503Error(AcunetixAPIError):
    pass


class Acunetix504Error(AcunetixAPIError):
    pass


class Acunetix429Error(AcunetixAPIError):
    pass


class Acunetix403Error(AcunetixAPIError):
    pass


class Acunetix409Error(AcunetixAPIError):
    pass


class Acunetix422Error(AcunetixAPIError):
    pass


class Acunetix502Error(AcunetixAPIError):
    pass


ERROR_MAP = {
    404: Acunetix404Error,
    401: Acunetix401Error,
    400: Acunetix400Error,
    500: Acunetix500Error,
    503: Acunetix503Error,
    504: Acunetix504Error,
    429: Acunetix429Error,
    403: Acunetix403Error,
    409: Acunetix409Error,
    422: Acunetix422Error,
    502: Acunetix502Error,
}



import io
import os
import tempfile
import typing
from urllib import request
import aiohttp
from aiofiles import open as aopen

from ..errors import Acunetix404Error
from ..schema import Report, ReportTemplate, Scan, Target, TypedList
from ..typehints import (
    CallbackFunction,
    InputReport,
    InputReportTemplate,
    InputScan,
    InputTargetType,
)
from ..utils import (
    get_input_report_id,
    get_input_report_template_id,
    get_input_target_id,
    run_sync,
)


class Reports:
    async def get_reports(self) -> typing.List[Report]:
        """
        Get all reports
        :return: List of `Report` objects
        :example:
        ```python
            >>> reports = await api.reports.get_reports()
            >>> for report in reports:
            >>>     print(report)
        ```
        """
        return [
            Report.from_dict(report)
            for report in (await self._request("GET", "reports"))["reports"]
        ]

    async def get_report(
        self,
        report: typing.Union[InputReport, InputScan, InputTargetType],
        input_type: typing.Optional[str] = None,
    ) -> Report:
        """
        Get report by ID
        :param report: Report ID, Scan ID, Target ID,
                       `Report`, `Scan` or `Target` object
        :param input_type: Type of input. Can be `report`, `scan` or `target`
                           This param will help to determine where to search
                           for the report
        :return: `Report` object
        :example:
        ```python
            >>> report = await api.reports.get_report(
                    "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                )
            >>> print(report)
        ```
        """
        if isinstance(report, (Report, str)):
            try:
                return Report.from_dict(
                    await self._request("GET", f"reports/{get_input_report_id(report)}")
                )
            except Acunetix404Error:
                if not isinstance(report, str) or input_type and input_type == "report":
                    raise

        if isinstance(report, (Scan, str)):
            try:
                scan: Scan = await self.get_scan(report)
            except Acunetix404Error:
                if not isinstance(report, str) or input_type and input_type == "scan":
                    raise

            report: InputTargetType = scan.target

        if isinstance(report, (Target, str)):
            reports: typing.List[Report] = await self.get_reports()
            for report in reports:
                if get_input_target_id(report) in report.source.id_list:
                    return report

        raise Acunetix404Error(404, "Report not found")

    async def create_report(
        self,
        template: InputReportTemplate,
        source: TypedList,
        done_callback: typing.Optional[CallbackFunction] = None,
    ) -> Report:
        """
        Create new report
        :param template: Report template ID or `ReportTemplate` object
        :param source: `TypedList` object
        :param done_callback: Callback function that will be called
                              when report is ready
        :return: `Report` object
        :example:
        ```python
            >>> report = await api.reports.create_report(
                    "11111111-1111-1111-1111-111111111111",
                    TypedList(
                        "scans",
                        [
                            "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                            "316f58ff-f6d6-47d5-b5e3-806837a8cfe3",
                        ],
                    ),
                )
            >>> print(report)
        ```
        """
        response: dict = await self._request(
            "POST",
            "reports",
            {"template_id": get_input_report_template_id(template), "source": source},
        )

        if "report_id" not in response:
            raise ValueError("Invalid response")

        if done_callback:
            self._callbacks[response["report_id"]] = done_callback

        return await self.get_report(response["report_id"])

    async def get_report_templates(self) -> typing.List[ReportTemplate]:
        """
        Get all report templates
        :return: List of `ReportTemplate` objects
        :example:
        ```python
            >>> report_templates = await api.reports.get_report_templates()
            >>> for report_template in report_templates:
            >>>     print(report_template)
        ```
        """
        return [
            ReportTemplate.from_dict(report_template)
            for report_template in (await self._request("GET", "report_templates"))[
                "report_templates"
            ]
        ]

    async def delete_reports(self, reports: typing.List[InputReport]) -> None:
        """
        Delete reports
        :param reports: List of report IDs or `Report` objects
        :return: None
        :example:
        ```python
            >>> await api.reports.delete_reports(
                    [
                        "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                        Report(...),
                    ],
                )
        ```
        """
        await self._request(
            "POST",
            "reports/delete",
            {"report_id_list": [get_input_report_id(report) for report in reports]},
        )

    async def delete_report(self, report: InputReport) -> None:
        """
        Delete report
        :param report: Report ID or `Report` object
        :return: None
        :example:
        ```python
            >>> await api.reports.delete_report(
                    "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                )
            >>> await api.reports.delete_report(
                    Report(...),
                )
        ```
        """
        await self.delete_reports([report])

    async def download_report(self, report: InputReport) -> typing.List[io.BytesIO]:
        """
        Download report
        :param report: Report ID or `Report` object
        :return: Report content
        :example:
        ```python
            >>> report_content = await api.reports.download_report(
                    "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                )
            >>> print(report_content)
        ```
        """
        report: Report = await self.get_report(report)
        files: typing.List[io.BytesIO] = []

        for uri in report.download:
            report_url = f"https://{self._endpoint}{uri}"
            with tempfile.TemporaryDirectory() as tmpdir:
                file = os.path.join(tmpdir, "report.pdf")
                async with aopen(file, "wb") as f:
                    async with aiohttp.ClientSession() as session:
                        async with session.get(report_url) as resp:
                            async for chunk in resp.content.iter_chunked(10):
                                await f.write(chunk)
                
                async with aopen(file, "rb") as f:
                    file: io.BytesIO = io.BytesIO(await f.read())

                file.name = await run_sync(
                    lambda: (
                        request.urlopen(request.Request(report_url))
                        .info()
                        .get_filename()
                    )
                )
                file.seek(0)
                files.append(file)

        return files



import dataclasses
import typing


@dataclasses.dataclass
class ScanProfile:
    profile_id: str
    checks: typing.List[str]
    custom: bool
    name: str
    sort_order: int

    @classmethod
    def from_dict(cls, data: dict) -> "ScanProfile":
        return cls(**data)

    def to_json(self) -> dict:
        return dataclasses.asdict(self)


FULL_SCAN = ScanProfile(
    "11111111-1111-1111-1111-111111111111",
    [],
    False,
    "Full Scan",
    10,
)

HIGH_RISK_VULNERABILITIES = ScanProfile(
    "11111111-1111-1111-1111-111111111112",
    [],
    False,
    "High Risk Vulnerabilities",
    20,
)

HIGH_OR_MEDIUM_RISK_VULNERABILITIES = ScanProfile(
    "11111111-1111-1111-1111-111111111119",
    [],
    False,
    "High or Medium Risk Vulnerabilities",
    30,
)

CROSS_SITE_SCRIPTING_VULNERABILITIES = ScanProfile(
    "11111111-1111-1111-1111-111111111116",
    [],
    False,
    "Cross-site Scripting Vulnerabilities",
    40,
)

SQL_INJECTION_VULNERABILITIES = ScanProfile(
    "11111111-1111-1111-1111-111111111113",
    [],
    False,
    "SQL Injection Vulnerabilities",
    50,
)

WEAK_PASSWORDS = ScanProfile(
    "11111111-1111-1111-1111-111111111115",
    [],
    False,
    "Weak Passwords",
    60,
)

CRAWL_ONLY = ScanProfile(
    "11111111-1111-1111-1111-111111111117",
    [],
    False,
    "Crawl Only",
    70,
)

MALWARE_SCAN = ScanProfile(
    "11111111-1111-1111-1111-111111111120",
    [],
    False,
    "Malware Scan",
    90,
)



import dataclasses
import typing

from .current_session import CurrentSession
from .scheduling_options import SchedulingOptions
from .target import Target


@dataclasses.dataclass
class Scan:
    criticality: int
    current_session: CurrentSession
    incremental: bool
    max_scan_time: int
    profile_id: str
    profile_name: str
    scan_id: str
    schedule: SchedulingOptions
    target: Target
    target_id: str
    report_template_id: typing.Optional[str] = None
    next_run: typing.Optional[int] = None
    manual_intervention: typing.Optional[bool] = None

    @classmethod
    def from_dict(cls, data: dict):
        data["current_session"] = CurrentSession.from_dict(data["current_session"])
        data["schedule"] = SchedulingOptions.from_dict(data["schedule"])
        data["target"] = Target.from_dict(data["target"])
        return cls(**data)

    def to_json(self):
        return dataclasses.asdict(self)



import asyncio
import functools
import typing

from .schema import Report, ReportTemplate, Scan, ScanProfile, Target, User


def get_input_user_id(user: typing.Union[User, str]) -> str:
    """
    Get user ID from `User` object or string
    :param user: `User` object or string with user ID
    :return: User ID
    :example:
    ```python
        >>> user_id = get_input_user_id(
                "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
            )
        >>> print(user_id)
        >>> user_id = get_input_user_id(User(...))
        >>> print(user_id)
    ```
    """
    return user.user_id if isinstance(user, User) else user


def get_input_target_id(target: typing.Union[Target, str]) -> str:
    """
    Get target ID from `Target` object or string
    :param target: `Target` object or string with target ID
    :return: Target ID
    :example:
    ```python
        >>> target_id = get_input_target_id(
                "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
            )
        >>> print(target_id)
        >>> target_id = get_input_target_id(Target(...))
        >>> print(target_id)
    ```
    """
    return target.target_id if isinstance(target, Target) else target


def get_input_scan_id(scan: typing.Union[Scan, str]) -> str:
    """
    Get scan ID from `Scan` object or string
    :param scan: `Scan` object or string with scan ID
    :return: Scan ID
    :example:
    ```python
        >>> scan_id = get_input_scan_id(
                "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
            )
        >>> print(scan_id)
        >>> scan_id = get_input_scan_id(Scan(...))
        >>> print(scan_id)
    ```
    """
    return scan.scan_id if isinstance(scan, Scan) else scan


def get_input_report_template_id(
    report_template: typing.Union[ReportTemplate, str]
) -> str:
    """
    Get report template ID from `ReportTemplate` object or string
    :param report_template: `ReportTemplate` object or string with report template ID
    :return: Report template ID
    :example:
    ```python
        >>> report_template_id = get_input_report_template_id(
                "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
            )
        >>> print(report_template_id)
        >>> report_template_id = get_input_report_template_id(ReportTemplate(...))
        >>> print(report_template_id)
    ```
    """
    return (
        report_template.template_id
        if isinstance(report_template, ReportTemplate)
        else report_template
    )


def get_input_report_id(report: typing.Union[Report, str]) -> str:
    """
    Get report ID from `Report` object or string
    :param report: `Report` object or string with report ID
    :return: Report ID
    :example:
    ```python
        >>> report_id = get_input_report_id(
                "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
            )
        >>> print(report_id)
        >>> report_id = get_input_report_id(Report(...))
        >>> print(report_id)
    ```
    """
    return report.report_id if isinstance(report, Report) else report


def get_input_scan_profile_id(scan_profile: typing.Union[ScanProfile, str]) -> str:
    """
    Get scan profile ID from `ScanProfile` object or string
    :param scan_profile: `ScanProfile` object or string with scan profile ID
    :return: Scan profile ID
    :example:
    ```python
        >>> scan_profile_id = get_input_scan_profile_id(
                "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
            )
        >>> print(scan_profile_id)
        >>> scan_profile_id = get_input_scan_profile_id(ScanProfile(...))
        >>> print(scan_profile_id)
    ```
    """
    return (
        scan_profile.profile_id
        if isinstance(scan_profile, ScanProfile)
        else scan_profile
    )


# https://github.com/hikariatama/Hikka/blob/fd918af4f4a9a91dac04f05ad643956b9b709059/hikka/utils.py#L310
def run_sync(func, *args, **kwargs) -> asyncio.Future:
    """
    Run a non-async function in a new thread and return an awaitable
    :param func: Sync-only function to execute
    :return: Awaitable coroutine
    """
    return asyncio.get_event_loop().run_in_executor(
        None,
        functools.partial(func, *args, **kwargs),
    )



import dataclasses
import typing


@dataclasses.dataclass
class AccessMap:
    access_all_groups: bool
    group_id_list: typing.List[str]

    @classmethod
    def from_dict(cls, data: dict) -> "AccessMap":
        return cls(**data)

    def to_json(self) -> dict:
        return dataclasses.asdict(self)



import dataclasses

from .access_map import AccessMap


@dataclasses.dataclass
class User:
    email: str
    first_name: str
    last_name: str
    role: str
    access: AccessMap
    enabled: bool
    locked: bool
    password: str
    totp_enabled: bool
    user_id: str

    @classmethod
    def from_dict(cls, data: dict) -> "User":
        data["access"] = AccessMap.from_dict(data["access"])
        return cls(**data)

    def to_json(self) -> dict:
        return dataclasses.asdict(self)



import hashlib
import typing

from ..schema import User
from ..utils import get_input_user_id


class Users:
    async def get_users(self) -> typing.List[User]:
        """
        Get all users
        :return: List of `User` objects
        :example:
        ```python
            >>> users = await api.users.get_users()
            >>> for user in users:
            >>>     print(user)
        ```
        """
        return [
            User.from_dict(user)
            for user in (await self._request("GET", "users"))["users"]
        ]

    async def get_user(self, user: typing.Union[User, str]) -> User:
        """
        Get user by ID
        :param user_id: User ID or `User` object
        :return: `User` object
        :example:
        ```python
            >>> user = await api.users.get_user(
                    "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                )
            >>> print(user)
        ```
        """
        return User.from_dict(
            await self._request("GET", f"users/{get_input_user_id(user)}")
        )

    async def create_user(
        self,
        first_name: str,
        last_name: str,
        role: str,
        email: str,
        password: str,
    ) -> User:
        """
        Create new user
        :param username: Username
        :param password: Password
        :param email: Email
        :param role: Role
        :return: `User` object
        :example:
        ```python
            >>> user = await api.users.create_user(
                    "John",
                    "Doe",
                    "platform_admin",
                    "login@example.com",
                    "p@55w0rd",
                )
            >>> print(user)
        ```
        """
        return User.from_dict(
            await self._request(
                "POST",
                "users",
                json={
                    "first_name": first_name,
                    "last_name": last_name,
                    "role": role,
                    "email": email,
                    "password": hashlib.sha256(password.encode()).hexdigest(),
                },
            )
        )

    async def disable_users(
        self,
        users_list: typing.List[typing.Union[User, str]],
    ) -> None:
        """
        Disable users by IDs
        :param users_list: User ID list or `User` object list
        :return: None
        :example:
        ```python
            >>> await api.users.disable_users(
                    [
                        "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                        User(...),
                    ],
                )
        ```
        """
        await self._request(
            "POST",
            "users/disable",
            json={"user_id_list": [get_input_user_id(user) for user in users_list]},
        )

    async def enable_users(
        self,
        users_list: typing.List[typing.Union[User, str]],
    ) -> None:
        """
        Enable users by IDs
        :param users_list: User ID list or `User` object list
        :return: None
        :example:
        ```python
            >>> await api.users.enable_users(
                    [
                        "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                        User(...),
                    ],
                )
        ```
        """
        await self._request(
            "POST",
            "users/enable",
            json={"user_id_list": [get_input_user_id(user) for user in users_list]},
        )

    async def delete_users(
        self,
        users_list: typing.List[typing.Union[User, str]],
    ) -> None:
        """
        Delete users by IDs
        :param users_list: User ID list or `User` object list
        :return: None
        :example:
        ```python
            >>> await api.users.delete_users(
                    [
                        "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                        User(...),
                    ],
                )
        ```
        """
        await self._request(
            "POST",
            "users/delete",
            json={"user_id_list": [get_input_user_id(user) for user in users_list]},
        )

    async def disable_user(self, user: typing.Union[User, str]) -> None:
        """
        Disable user by ID
        :param user_id: User ID
        :return: None
        :example:
        ```python
            >>> await api.users.disable_user(
                    "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                )
            >>> await api.users.disable_user(User(...))
        ```
        """
        return await self.disable_users([user])

    async def enable_user(self, user: typing.Union[User, str]) -> None:
        """
        Enable user by ID
        :param user_id: User ID
        :return: None
        :example:
        ```python
            >>> await api.users.enable_user(
                    "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                )
            >>> await api.users.enable_user(User(...))
        ```
        """
        return await self.enable_users([user])

    async def delete_user(self, user: typing.Union[User, str]) -> None:
        """
        Delete user by ID
        :param user_id: User ID
        :return: None
        :example:
        ```python
            >>> await api.users.delete_user(
                    "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                )
            >>> await api.users.delete_user(User(...))
        ```
        """
        return await self.delete_users([user])

    async def update_user(
        self,
        user: typing.Union[User, str],
        first_name: str,
        last_name: str,
        role: str,
        email: str,
    ) -> User:
        """
        Update user
        :param user: User ID or `User` object
        :param username: Username
        :param email: Email
        :param role: Role
        :return: `User` object
        :example:
        ```python
            >>> user = await api.users.update_user(
                    "316f58ff-f6d6-47d5-b5e3-806837a8cfe2",
                    "John",
                    "Doe",
                    "platform_admin",
                    "login@example.com",
                )
            >>> print(user)
        ```
        """

        await self._request(
            "PATCH",
            f"users/{get_input_user_id(user)}",
            json={
                "first_name": first_name,
                "last_name": last_name,
                "role": role,
                "email": email,
            },
        )

        return await self.get_user(user)



import dataclasses
import typing


@dataclasses.dataclass
class TypedList:
    list_type: str
    id_list: typing.List[str]
    description: typing.Optional[str] = None

    @classmethod
    def from_dict(cls, data: dict):
        return cls(**data)

    def to_json(self):
        data = dataclasses.asdict(self)

        if data["description"] is None:
            data.pop("description")

        return data



import dataclasses
import datetime
import typing


@dataclasses.dataclass
class Target:
    address: str
    criticality: int
    description: str
    type: str
    continuous_mode: typing.Optional[bool] = None
    default_scanning_profile_id: typing.Optional[str] = None
    deleted_at: typing.Optional[datetime.datetime] = None
    fqdn: typing.Optional[str] = None
    fqdn_hash: typing.Optional[str] = None
    fqdn_status: typing.Optional[str] = None
    fqdn_tm_hash: typing.Optional[str] = None
    issue_tracker_id: typing.Optional[str] = None
    last_scan_date: typing.Optional[datetime.datetime] = None
    last_scan_id: typing.Optional[str] = None
    last_scan_session_id: typing.Optional[str] = None
    last_scan_session_status: typing.Optional[str] = None
    manual_intervention: typing.Optional[bool] = None
    severity_counts: typing.Optional[dict] = None
    target_id: typing.Optional[str] = None
    threat: typing.Optional[int] = None
    verification: typing.Optional[str] = None

    @classmethod
    def from_dict(cls, data: dict):
        if data.get("deleted_at"):
            data["deleted_at"] = datetime.datetime.fromisoformat(data["deleted_at"])
        if data.get("last_scan_date"):
            data["last_scan_date"] = datetime.datetime.fromisoformat(
                data["last_scan_date"]
            )
        return cls(**data)

    def to_json(self):
        data = dataclasses.asdict(self)

        if data["deleted_at"]:
            data["deleted_at"] = data["deleted_at"].isoformat()

        if data["last_scan_date"]:
            data["last_scan_date"] = data["last_scan_date"].isoformat()

        return data



import dataclasses
import datetime


@dataclasses.dataclass
class CurrentSession:
    event_level: int
    progress: int
    scan_session_id: str
    severity_counts: dict
    start_date: datetime.datetime
    status: str
    threat: int

    @classmethod
    def from_dict(cls, data: dict):
        data["start_date"] = datetime.datetime.fromisoformat(data["start_date"])
        return cls(**data)

    def to_json(self):
        data = dataclasses.asdict(self)
        data["start_date"] = data["start_date"].isoformat()
        return data



import dataclasses
import typing


@dataclasses.dataclass
class ReportTemplate:
    template_id: str
    group: str
    name: str
    accepted_sources: typing.Optional[typing.List[str]] = None

    @classmethod
    def from_dict(cls, data: dict) -> "ReportTemplate":
        return cls(**data)

    def to_json(self) -> dict:
        data = dataclasses.asdict(self)

        if data["accepted_sources"] is None:
            data.pop("accepted_sources")

        return data


DEVELOPER = ReportTemplate(
    "11111111-1111-1111-1111-111111111111",
    "Standart Reports",
    "Developer",
)

QUICK = ReportTemplate(
    "11111111-1111-1111-1111-111111111112",
    "Standart Reports",
    "Quick",
)

EXECUTIVE_SUMMARY = ReportTemplate(
    "11111111-1111-1111-1111-111111111113",
    "Standart Reports",
    "Executive Summary",
)

AFFECTED_ITEMS = ReportTemplate(
    "11111111-1111-1111-1111-111111111115",
    "Standart Reports",
    "Affected Items",
)

COMPREHENSIVE = ReportTemplate(
    "11111111-1111-1111-1111-111111111126",
    "Standart Reports",
    "Comprehensive (New)",
)

HIPAA = ReportTemplate(
    "11111111-1111-1111-1111-111111111114",
    "Compliance Reports",
    "HIPAA",
)


SCAN_COMPARISON = ReportTemplate(
    "11111111-1111-1111-1111-111111111124",
    "Compliance Reports",
    "Scan Comparison",
)

CWE_SANS_TOP_25 = ReportTemplate(
    "11111111-1111-1111-1111-111111111116",
    "Compliance Reports",
    "CWE 2011",
)

ISO_27001 = ReportTemplate(
    "11111111-1111-1111-1111-111111111117",
    "Compliance Reports",
    "ISO 27001",
)

NIST_SP800_53 = ReportTemplate(
    "11111111-1111-1111-1111-111111111118",
    "Compliance Reports",
    "NIST SP800 53",
)

OWASP_TOP_10_2013 = ReportTemplate(
    "11111111-1111-1111-1111-111111111119",
    "Compliance Reports",
    "OWASP Top 10 2013",
)

OWASP_TOP_10_2017 = ReportTemplate(
    "11111111-1111-1111-1111-111111111125",
    "Compliance Reports",
    "OWASP Top 10 2017",
)

OWASP_TOP_10_2021 = ReportTemplate(
    "11111111-1111-1111-1111-111111111127",
    "Compliance Reports",
    "OWASP Top 10 2021",
)

PCI_DSS_3_2 = ReportTemplate(
    "11111111-1111-1111-1111-111111111120",
    "Compliance Reports",
    "PCI DSS 3.2",
)

SARBANES_OXLEY = ReportTemplate(
    "11111111-1111-1111-1111-111111111121",
    "Compliance Reports",
    "Sarbanes Oxley",
)

STIG_DISA = ReportTemplate(
    "11111111-1111-1111-1111-111111111122",
    "Compliance Reports",
    "STIG DISA",
)

WASC_THREAT_CLASSIFICATION = ReportTemplate(
    "11111111-1111-1111-1111-111111111123",
    "Compliance Reports",
    "WASC Threat Classification",
)

TEMPLATE_MAP = {
    DEVELOPER.template_id: DEVELOPER,
    QUICK.template_id: QUICK,
    EXECUTIVE_SUMMARY.template_id: EXECUTIVE_SUMMARY,
    AFFECTED_ITEMS.template_id: AFFECTED_ITEMS,
    COMPREHENSIVE.template_id: COMPREHENSIVE,
    HIPAA.template_id: HIPAA,
    SCAN_COMPARISON.template_id: SCAN_COMPARISON,
    CWE_SANS_TOP_25.template_id: CWE_SANS_TOP_25,
    ISO_27001.template_id: ISO_27001,
    NIST_SP800_53.template_id: NIST_SP800_53,
    OWASP_TOP_10_2013.template_id: OWASP_TOP_10_2013,
    OWASP_TOP_10_2017.template_id: OWASP_TOP_10_2017,
    OWASP_TOP_10_2021.template_id: OWASP_TOP_10_2021,
    PCI_DSS_3_2.template_id: PCI_DSS_3_2,
    SARBANES_OXLEY.template_id: SARBANES_OXLEY,
    STIG_DISA.template_id: STIG_DISA,
    WASC_THREAT_CLASSIFICATION.template_id: WASC_THREAT_CLASSIFICATION,
}



import dataclasses
import datetime
import typing

from .notification_data import NotificationData


@dataclasses.dataclass
class Notification:
    consumed: bool
    created: datetime.datetime
    data: NotificationData
    email: typing.Optional[str]
    notification_id: str
    resource_id: str
    resource_type: int
    severity: int
    type_id: int
    user_id: typing.Optional[str]

    @classmethod
    def from_dict(cls, data: dict):
        data["created"] = datetime.datetime.fromisoformat(data["created"])
        data["data"] = NotificationData.from_dict(data["data"])
        return cls(**data)

    def to_json(self):
        data = dataclasses.asdict(self)
        data["created"] = data["created"].isoformat()
        return data



from setuptools import setup, find_packages
import os
import pathlib
import version

setup(
    name="acunetix",
    version=version.__version__,
    packages=find_packages(),
    long_description=pathlib.Path(
        os.path.join(os.path.dirname(__file__), "README.md")
    ).read_text(),
)



import dataclasses
import datetime
import typing


@dataclasses.dataclass
class SchedulingOptions:
    disable: bool
    time_sensitive: bool
    triggerable: bool
    history_limit: typing.Optional[int] = None
    recurrence: typing.Optional[str] = None
    start_date: typing.Optional[datetime.datetime] = None

    @classmethod
    def from_dict(cls, data: dict):
        if data.get("start_date"):
            data["start_date"] = datetime.datetime.fromisoformat(data["start_date"])
        return cls(**data)

    def to_json(self):
        data = dataclasses.asdict(self)
        if data["start_date"]:
            data["start_date"] = data["start_date"].isoformat()

        if not data["recurrence"]:
            data.pop("recurrence")

        if not data["history_limit"]:
            data.pop("history_limit")

        return data



import dataclasses


@dataclasses.dataclass
class FileAttachment:
    name: str
    path: str
    type: str

    @classmethod
    def from_dict(cls, data: dict):
        return cls(**data)

    def to_json(self):
        return dataclasses.asdict(self)



import dataclasses
import datetime
import typing

from .file_attachment import FileAttachment


@dataclasses.dataclass
class NotificationData:
    ended: typing.Optional[datetime.datetime] = None
    started: typing.Optional[datetime.datetime] = None
    name: typing.Optional[str] = None
    source: typing.Optional[str] = None
    creator: typing.Optional[str] = None
    source_type: typing.Optional[str] = None
    template_name: typing.Optional[str] = None
    target_id: typing.Optional[str] = None
    vuln_stats: typing.Optional[typing.List[int]] = None
    event_level: typing.Optional[int] = None
    target_desc: typing.Optional[typing.List[str]] = None
    profile_name: typing.Optional[str] = None
    scan_session_id: typing.Optional[str] = None
    failed_job_count: typing.Optional[int] = None
    file_attachments: typing.Optional[typing.List[typing.Dict[str, str]]] = None

    @classmethod
    def from_dict(cls, data: dict):
        if data.get("file_attachments"):
            data["file_attachments"] = [
                FileAttachment.from_dict(x) for x in data["file_attachments"]
            ]
        return cls(**data)

    def to_json(self):
        return dataclasses.asdict(self)



import dataclasses
import datetime
import typing

from .report_template import TEMPLATE_MAP, ReportTemplate
from .typed_list import TypedList


@dataclasses.dataclass
class Report:
    download: typing.List[str]
    generation_date: datetime.datetime
    report_id: str
    status: str
    source: TypedList
    template: typing.Optional[ReportTemplate] = None
    template_id: typing.Optional[str] = None
    template_name: typing.Optional[str] = None
    template_type: typing.Optional[int] = None

    def __post_init__(self):
        if self.template:
            if self.template_id is None:
                self.template_id = self.template.template_id

            if self.template_name is None:
                self.template_name = self.template.name

            if self.template_type is None:
                self.template_type = 0

    @classmethod
    def from_dict(cls, data: dict):
        data["generation_date"] = datetime.datetime.fromisoformat(
            data["generation_date"]
        )
        data["source"] = TypedList.from_dict(data["source"])
        data.setdefault(
            "template",
            TEMPLATE_MAP.get(data.get("template_id", None), None),
        )

        return cls(**data)

    def to_json(self):
        data = dataclasses.asdict(self)
        data["generation_date"] = data["generation_date"].isoformat()
        data.pop("template", None)
        return data
